// ScreenSizer Module
// Jeffrey J Weimer
// SVN History: $Revision: 789 $ on $Date: 2008-06-16 12:00:04 -0500 (Mon, 16 Jun 2008) $
//
// See the Help file for further information

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.4

#pragma ModuleName=ScreenSizer

// Constants

Static StrConstant kcmdMFW="ScreenSizer#MoveFrontWindow"

// Initialize
//   call to the function as ScreenSizer#Initialize(winXX)
//	winXX: 0 - do not change frame size on WinXX systems (default!)
//	winXX: 1 - use full frame size on WinXX systems

Static Function Initialize(winXX)
	variable winXX

	if ((winXX!=0) || (winXX!=1))
		winXX = 0
	endif
	
	if (DataFolderExists("root:Packages:ScreenSizer")==0)
	
		// initalize from clean start
	
		string cdf = GetDataFolder(1)
		
		NewDataFolder/O/S root:Packages
		NewDataFolder/O/S ScreenSizer
		variable/G scrwpxl, scrhpxl, scrwpnt, scrhpnt
		variable/G scrleft = 0, scrtop = 0, scrright = 100, scrbottom = 100
		SetDataFolder $cdf

	endif

	NVAR scrwx = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhx = root:Packages:ScreenSizer:scrhpxl
	NVAR scrwn = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhn = root:Packages:ScreenSizer:scrhpnt

	if (cmpstr(IgorInfo(2),"Macintosh")==0)

		string IgorStuff=""
		variable scr0,scr1,scr2
	
		IgorStuff=IgorInfo(0)
		scr0 = strsearch(IgorStuff,"RECT",0)
		scr1 = strsearch(IgorStuff,",",scr0+9)
		scr2 = strlen(IgorStuff)-2
		
		// values are in RELATIVE PIXELS (for tables and graphs)!!!!
		
		scrwx = str2num(IgorStuff[scr0+9,scr1-1])/100
		scrhx = str2num(IgorStuff[scr1+1,scr2])/100
	else

		// maximize the frame
		
		if (winXX)
			MoveWindow/F 2, 2, 2, 2
		endif
		GetWindow kwFrameInner wsizeDC

		// values are in RELATIVE PIXELS (for panels)!!!!
		
		scrwx = (v_right-v_left)/100
		scrhx = (v_bottom-v_top)/100
	endif
	
	// values in RELATIVE POINTS (for tables and graphs)
	
	scrwn = scrwx*72/ScreenResolution
	scrhn = scrhx*72/ScreenResolution
	
	return 0
End

// Set Frame

Static Function SetFrame(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom
	
	scrleft = left
	scrtop = top
	scrright = right
	scrbottom = bottom
	return 0
end

// Move Front Window

Static Function MoveFrontWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=round(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=round(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=round(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=round(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	MoveWindow left,top,right,bottom
	return 0
end

// Move Command Window

Static Function MoveCmdWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=round(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=round(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=round(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=round(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	MoveWindow/C left,top,right,bottom
	return 0
end

// Make a Panel

Static Function MakePanel(left, top, width, height, strname, [other])
	variable left, top, width, height
	string strname, other
	
	variable right, bottom
	string exeCmd
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl

	if (left>=0)
		left=round(scrwpxl*(scrleft + (scrright - scrleft)*left/100))
	else
		left=round(scrwpxl*(scrright + (scrright - scrleft)*left/100)) - width
	endif
	if (top>=0)
		top=round(scrhpxl*(scrtop + (scrbottom-scrtop)*top/100))
	else
		top=round(scrhpxl*(scrbottom + (scrbottom-scrtop)*top/100)) - height
	endif
	right=left+width
	bottom=top+height
	
	if (ParamIsDefault(other))
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)as \"%s\"", left, top, right, bottom, strname
	else
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	endif
	execute/Q/Z exeCmd
	return 0
	
end

// Make an External Panel

Static Function MakeExtPanel(side, offset, width, height, strname, [other])
	variable side, offset, width, height
	string strname, other
	
	variable left, right, top, bottom
	string exeCmd
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl

	switch(side)
		case 0:		
		case 1:
			left = offset
			top = 0
			right = width+offset
			bottom = height
			break
		default:
			left = 0
			top = offset
			right = width
			bottom = height+offset
			break
	endswitch
	
	if (ParamIsDefault(other))
		other = "/HOST=#"
	endif
	other += "/EXT=" + num2str(side)
	sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	execute/Q/Z exeCmd
	return 0
	
end

// Save Window Coordinates

Static Function SaveWinCoords([name])
	string name
	
	variable left, top, right, bottom
	string cdf = GetDataFolder(1), theCoords=""
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif
	switch(WinType(name))
		case 0:
			return -1
			break
		case 13:
			return -1
			break
		default:
			GetWindow $name wsize
			left = round(((V_left/scrwpnt) - scrleft)*1000/(scrright-scrleft))/10
			top = round(((V_top/scrhpnt) - scrtop)*1000/(scrbottom-scrtop))/10
			right = round(((V_right/scrwpnt) - scrleft)*1000/(scrright-scrleft))/10
			bottom = round(((V_bottom/scrhpnt) - scrtop)*1000/(scrbottom-scrtop))/10
			sprintf theCoords, "(%g,%g,%g,%g)", left, top, right, bottom
			SetWindow $name userData(ScreenSizer)=theCoords
			break
	endswitch
	return 0
end

// Restore Window Coordinates
// how:
//	0 (default) - move only; no VISIBLE shrink/expand; possibly go outside boundary
//	1 - move; no VISIBLE shrink/expand; move back inside boundary
//	2 - move; VISIBLE shrink/expand (thereby stay inside boundary)

Static Function RestoreWinCoords(how,[name])
	variable how
	string name
	
	string theCmd, theLen
	variable left, top, right, bottom, offset
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt
	
	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif
	switch(WinType(name))
		case 0:
			return -1
		case 13:
			return -1
		case 7:	// panels have left + top + WIDTH + HEIGHT
			theCmd = GetUserData(name,"","ScreenSizer")
			if (strlen(theCmd)==0)
				return -1
			endif
			GetWindow $name wsize
			theLen = StringFromList(0,theCmd,",")
			left = str2num(theLen[1,strlen(theCmd)-1])
			top = str2num(StringFromList(1,theCmd,","))
			right = left + round(1000*(V_right - V_left)/(scrwpnt*(scrright-scrleft)))/10
			bottom = top + round(1000*(V_bottom - V_top)/(scrhpnt*(scrbottom-scrtop)))/10
			if (how>0)
				if (right>100)
					offset = 100 - right
					if ((left+offset)>=0)
						left += offset
						right += offset
					else
						right += -left
						left = 0 
					endif
				endif
				if (bottom>100)
					offset = 100 - bottom
					if ((top+offset)>=0)
						top += offset
						bottom += offset
					else
						bottom += -top
						top = 0
					endif
				endif
			endif
			sprintf theCmd, "%s(%g,%g,%g,%g)", kcmdMFW, left, top, right, bottom
			break
		default:
			theCmd = GetUserData(name,"","ScreenSizer")
			if (strlen(theCmd)==0)
				return -1
			endif
			GetWindow $name wsize
			switch(how)
				case 0:
					theLen = StringFromList(0,theCmd,",")
					left = str2num(theLen[1,strlen(theLen)-1])
					top = str2num(StringFromList(1,theCmd,","))
					right = left + round(1000*(V_right - V_left)/(scrwpnt*(scrright-scrleft)))/10
					bottom = top + round(1000*(V_bottom - V_top)/(scrhpnt*(scrbottom-scrtop)))/10
					sprintf theCmd, "%s(%g,%g,%g,%g)", kcmdMFW, left, top, right, bottom
					break
				case 1:
					theLen = StringFromList(0,theCmd,",")
					left = str2num(theLen[1,strlen(theCmd)-1])
					top = str2num(StringFromList(1,theCmd,","))
					right = left + round(1000*(V_right - V_left)/(scrwpnt*(scrright-scrleft)))/10
					bottom = top + round(1000*(V_bottom - V_top)/(scrhpnt*(scrbottom-scrtop)))/10
					if (right>100)
						offset = 100 - right
						if ((left+offset)>=0)
							left += offset
							right += offset
						else
							right += -left
							left = 0 
						endif
					endif
					if (bottom>100)
						offset = 100 - bottom
						if ((top+offset)>=0)
							top += offset
							bottom += offset
						else
							bottom += -top
							top = 0
						endif
					endif
					sprintf theCmd, "%s(%g,%g,%g,%g)", kcmdMFW, left, top, right, bottom
					break
				case 2:
					sprintf theCmd, "%s%s", kcmdMFW, theCmd
					break
			endswitch
			break
	endswitch
	DoWindow/F $name
	print name, theCmd
	Execute/Q/Z theCmd
	return 0
end

Static Function SaveAllWindows()

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt = ItemsInList(theList)
	for(ic=nt;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		SaveWinCoords(name=theOne)
	endfor
	return 0
end

Static Function RestoreAllWindows(how)
	variable how

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt=ItemsInList(theList)
	for(ic=nt-1;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		RestoreWinCoords(how,name=theOne)
	endfor
	return 0
end

// Percentage of Screen Width left After Offset in piXels

Static Function pWaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrright = root:Packages:ScreenSizer:scrright

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	
	variable rtV
	
	 rtV = ((scrright - scrleft)*scrwpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end

// Percentage of Screen Height left After Offset in piXels

Static Function pHaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	
	variable rtV
	
	 rtV = ((scrbottom - scrtop)*scrhpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end
